<?php
/**
 * Created by PhpStorm.
 * User: eugeniofonseca
 */

namespace Service;


use Util\InstanceProvider;
use Util\Messages;
use Util\PHPWord\PMTemplateProcessor;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Remote\FileUploader\Server as ServerFileUploader;

class DocumentTemplate {

    public function findDocumentTemplate($conditions,$page,$pageSize) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditionsSQL = Utilities::createQueryFiltered($conditions);
        $documentTemplates = InstanceProvider::getRepository()->getDocumentTemplate()->getDocumentTemplate(
            InstanceProvider::getEnvironment()->getCurrentOrganizationId(), $conditionsSQL, $limit,
            $offset);
        return $documentTemplates;
    }

    public function removeDocumentTemplate($documentTemplateId){
        $entityManager = InstanceProvider::getEntityManager();
        $documentTemplate = InstanceProvider::getRepository()->getDocumentTemplate()->find($documentTemplateId);
        $entityManager->remove($documentTemplate);
        $entityManager->flush();
    }

    public function saveDocumentTemplate($documentTemplateId, $name, $type, $documents = array())
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $documentTemplate = null;
            if ($documentTemplateId) {
                /** @var  Entities\DocumentTemplate $documentTemplate */
                $documentTemplate = InstanceProvider::getRepository()->getDocumentTemplate()->find($documentTemplateId);
            }
            $documentTemplate = InstanceProvider::getFactory()->getDocumentTemplate()->save($documentTemplate, $name, $type);


            if (!empty($documents)) {
                $attachmentService = InstanceProvider::getService()->getAttachment();
                $attachment = $attachmentService->createDocuments($documents)[0];
                $oldAttachment = $documentTemplate->getAttachment();
                $documentTemplate->setAttachment($attachment);
                $entityManager->persist($documentTemplate);
                $entityManager->flush();
                if($oldAttachment){
                    $attachmentService->deleteAttachmentByEntities([$oldAttachment]);
                }
            }

            $entityManager = InstanceProvider::getEntityManager();
            $entityManager->persist($documentTemplate);
            $entityManager->flush();
            $connection->commit();
            return $documentTemplate->getId();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    public function getDocumentBudget($documentTemplateId,$issueId){
        /** @var Entities\Budget $budgets */
        $budgets = InstanceProvider::getRepository()->getBudget()->findOneBy(array('task'=> $issueId));
        if ($budgets && $budgets->getExternalId()) {

            $linkDownloadBudgetSm = InstanceProvider::getService()->getIssue()->linkDownloadBudgetSm($issueId);
            if (!empty($linkDownloadBudgetSm)) {
                $linkDownloadBudgetSm = $linkDownloadBudgetSm[0];
            }
            $this->downloadBudgetSm($linkDownloadBudgetSm);
            /** @var ServerFileUploader $zhFileUploader */
            $zhFileUploader = InstanceManager::getInstance()->getService('fileUploader');
            $zhFileUploader->deleteFile($linkDownloadBudgetSm);
        } else {
            throw new \Exception('Só é possível gerar o orçamento para finalizar a especificação');
        }


    }

    public function downloadBudgetSm($linkDocument)
    {
        if ($linkDocument) {
            $type = "application/pdf";
            $name = "Orçamento.pdf";
            $this->setParamsForDownload($type, null, $name, $linkDocument,false);
            exit;
        } else {
            throw new \Exception('ERR_NOT_ATTACHMENT');
        }
    }

    public function setParamsForDownload($type, $size, $name, $fileName,$isAttachment = true)
    {
        header('Accept-Ranges: none');
        header("Content-Type: " . $type);
        if($size){
            header("Content-Length: " . $size);
        }

        if($isAttachment){
            header('Content-Disposition: attachment; filename="' . $name.'"');
        }else{
            header('Content-Disposition: inline;filename="' . $name.'"');
        }
        readfile($fileName);
    }

    public function generateBudget($documentTemplateId,$issueId)
    {
        /** @var Entities\DocumentTemplate $documentTemplate */
        $documentTemplate = InstanceProvider::getRepository()->getDocumentTemplate()->find($documentTemplateId);
        $attachment = $documentTemplate->getAttachment();
        $tempDir =Utilities::getTemporaryDir();
        $ds = DIRECTORY_SEPARATOR;
        $urlFile = Utilities::getFileServerFilesUrl().$attachment->getFileName();
        $fileDocx = @file_get_contents($urlFile);
        $fileDirTemp = $tempDir . $ds . 'PM_'.$issueId.'_'.$attachment->getLabel();
        if ($fileDocx !== false) {
            if (is_dir($tempDir) && is_writable($tempDir)) {
                file_put_contents($fileDirTemp, $fileDocx);

            }
            else {
                throw new \Exception(Messages::ERR_GENERATION_DIR_PERMISSION_1 . $tempDir . Messages::ERR_GENERATION_DIR_PERMISSION_2);
            }
        }
        else {
            throw new \Exception(Messages::ERR_GENERATION_LOAD_URL . $urlFile . "\"");
        }
        if (is_file($fileDirTemp)) {

            $templateProcessor = new PMTemplateProcessor($fileDirTemp);

            $this->replaceBudgetDocumentData($issueId,$templateProcessor);

            $fileDirTempModify = $tempDir . $ds . 'PM_Budget_Issue_'.$issueId.'_'.$documentTemplate->getName().'.docx';

            $templateProcessor->saveAs($fileDirTempModify);
            $cmd = "sudo soffice --headless --convert-to pdf --outdir " . $tempDir . " \"" . $fileDirTempModify . "\" 2>&1";
            $resultCmd = shell_exec($cmd);

            if (strpos(strtolower($resultCmd), "failed") !== false ||
                strpos(strtolower($resultCmd), "error") !== false
            ) {
                throw new \Exception(Messages::ERR_GENERATION_PDF . $resultCmd);
            }

            $fileName = 'PM_Budget_Issue_'.$issueId.'_'.$documentTemplate->getName().'.pdf';
            $fileDirTempModifyPdf = $tempDir . $ds . $fileName;
            $size = filesize($fileDirTempModifyPdf);
            header('Accept-Ranges: none');
            header("Content-Type: application/pdf");
            header("Content-Length: " . $size);
            header("Content-Disposition: attachment; filename=" . $fileName);
            readfile($tempDir . $ds . $fileName);

            self::clearFileBudget(array($fileDirTemp, $fileDirTempModify, $fileDirTempModifyPdf));
            exit;
        }
    }

    public static function clearFileBudget(array $files = array())
    {
        foreach ($files as $file) {
            if (is_file($file)) {
                unlink($file);
            }
        }


    }

    public static function getMnemonicsInBudget(){
        return array("NAME_SPONSOR","DOCUMENT_NUMBER","BUDGET_ID","EMAIL","SUBTOTAL_VALUE","DISCOUNT_PERCENT",
            "DISCOUNT_VALUE","TOTAL_VALUE","DTNOW","ISSUE_ID","ISSUE_NAME","MANAGER_NAME","STRUCTURE_NAME",
            "ISSUE_DESCRIPTION","TOTAL_VALUE_WORDS");
    }

    private function setValuesWithSetValue(PMTemplateProcessor $templateProcessor, $values, $mnemonics)
    {
        foreach ($mnemonics as $mnemonic) {
            $templateProcessor->setValue('[' . $mnemonic . ']', $values[$mnemonic]);
        }
    }

    private function replaceBudgetDocumentData($issueId,PMTemplateProcessor $templateProcessor)
    {
        $basicValues = InstanceProvider::getRepository()->getDocumentTemplate()->getBasicBudgetDocumentData($issueId);
        $primaryValues = $basicValues[0];
        $primaryValues["DOCUMENT_NUMBER"] = Utilities::maskCpfAndCnpj($primaryValues["DOCUMENT_NUMBER"]);
        $primaryValues["TOTAL_VALUE_WORDS"] = Utilities::convertCurrencyToWords($primaryValues["TOTAL_VALUE"]);

        self::setValuesWithSetValue($templateProcessor,$primaryValues,self::getMnemonicsInBudget());
        $itemBudgetValues = InstanceProvider::getRepository()->getDocumentTemplate()->getItemBudgetDocumentData($issueId);

        $this->setTablesInDocuments($templateProcessor,$itemBudgetValues);

    }

    public function setTablesInDocuments( PMTemplateProcessor $templateProcessor, array $itemBudgetValues){
        $documentMain = $templateProcessor->getTempDocumentMainPart();

        $closeTag = "</w:tr>";

        $documentLength = strlen($documentMain);

        $indexInd = $this->getIndexMnemonicTable($documentMain);

        if ($indexInd !== false) {
            $closedTrIndex = $this->getIndexCloseTR($documentMain, $indexInd);
            list($startIndex, $endIndex) = $this->getIndexBeginAndEndTable($documentMain, $documentLength, $closedTrIndex);


            $startClob = substr($documentMain, 0, $endIndex + strlen($closeTag));
            $lineClob = substr($documentMain, $startIndex, $endIndex - $startIndex);
            $endClob = substr($documentMain, $endIndex, $documentLength);

            $qtdItens = count($itemBudgetValues);
            $itemBudgetMnemonics = self::getMnemonicsInItemBudget();
            foreach ($itemBudgetValues as $itemBudgetValue) {
                $lineReplacedClob = $this->setValuesWithReplace($lineClob,$itemBudgetValue,$itemBudgetMnemonics);
                $startClob = $startClob . $lineReplacedClob . $closeTag;
            }

            if ($qtdItens > 0) {
                $documentMain = $startClob . substr($endClob, strlen($closeTag));
            }

            $documentMain = str_replace($lineClob.$closeTag, "", $documentMain);
        }
        $templateProcessor->setTempDocumentMainPart($documentMain);
    }

    public static function getMnemonicsInItemBudget(){
        return array("ROWNUM","ITEM_NAME","QUANTITY","UNIT_MEASURE","ITEM_VALUE","TOTAL_VALUE_ITEM");
    }

    private function setValuesWithReplace($lineClob,$itemValues,$mnemonics) {
        $lineReplacedClob = str_replace("[IND]", "", $lineClob);
        foreach ($mnemonics as $mnemonic) {
            $value = '';
            if(!empty($itemValues[$mnemonic])){
                $value = $itemValues[$mnemonic];
            }
            $lineReplacedClob = str_replace('['.$mnemonic.']', $value, $lineReplacedClob);
        }

        return $lineReplacedClob;
    }

    public function getIndexMnemonicTable($documentMain)
    {
        $indexInd = strpos($documentMain, "[IND]");
        return $indexInd;
    }

    public function getIndexCloseTR($documentMain, $indexInd)
    {
        $closedTrIndex = strpos($documentMain, "</w:tr>", $indexInd);
        return $closedTrIndex;
    }

    public function getIndexBeginAndEndTable($documentMain, $documentLength, $closedTrIndex)
    {
        $endIndex = -1;
        $startIndex = -1;

        $firstTrIndex = strrpos($documentMain, "<w:tr ", (($documentLength - $closedTrIndex) * -1));

        if (!$firstTrIndex) {
            $firstTrIndex = strrpos($documentMain, "<w:tr>", (($documentLength - $closedTrIndex) * -1));
        }
        if ($firstTrIndex !== false) {
            $startIndex = $firstTrIndex;
            $endIndex = $closedTrIndex;
        }
        return array($startIndex, $endIndex);
    }
}
