<?php 
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities AS DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class ErrorClassification
{
    /**
     * @var EntityManager
     */
    private $entityManager;
    
    public function __construct(EntityManager $entityManager)
    {
         $this->entityManager = $entityManager;
    }
    
    public function findErrorClassification($conditions)
    {
        $errorClassificationRepository = InstanceProvider::getRepository()->getErrorClassification();
        $environment = InstanceProvider::getEnvironment();
        $alias = 'ECL';
        foreach($conditions as &$condition){
            if($condition['columnName']  == 'STATUS_FILTER'){
                $condition['columnName'] = 'STATUS';
            }
        }
        $filters = DevOpsUtilities::createQueryFiltered($conditions, $alias);
        $organizationId = $environment->getCurrentOrganizationId();
        $errorClassification = $errorClassificationRepository->findErrorClassification($organizationId, $filters);
        return $errorClassification;
    }
    
    public function saveErrorClassification($id, $description, $status)
    {
        $connection = $this->entityManager->getConnection();
        $errorClassificationRepository = InstanceProvider::getRepository()->getErrorClassification();
        $errorClassificationFactory = InstanceProvider::getFactory()->getErrorClassification();
        $errorClassificationEntity = NULL;
        try {
            $connection->beginTransaction();
            if($id){
                $errorClassificationEntity = $errorClassificationRepository->find($id);
            }
            $errorClassificationEntity = $errorClassificationFactory
                                         ->createErrorClassification($errorClassificationEntity, $description, $status);
            $this->entityManager->persist($errorClassificationEntity);
            $this->entityManager->flush();
            $connection->commit();
            return array("ID" => $errorClassificationEntity->getId());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deleteErrorClassification($errorClassificationId)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $errorClassificationRepository = InstanceProvider::getRepository()->getErrorClassification();
            $errorClassificationEntity = $errorClassificationRepository->find($errorClassificationId);
            $errorClassificationEntity->setStatus(Entities\ErrorClassification::DELETED);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Doctrine\DBAL\DBALException $e) {
            $connection->rollBack();
            throw new \Exception($e->getMessage(), 1);
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
}