<?php

namespace Zeedhi\DevOps\Model\Configurationmanager\Repositories;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskRelationship as TaskRelationshipEntities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Release extends \Doctrine\ORM\EntityRepository {
    /**
     * @param $productId
     * @param $organizationId
     * @return array
     */
    public function findPackageServersByProduct($productId, $organizationId = null, $filterString = '') {
        $query = <<<SQL
        SELECT * FROM (
			SELECT SV.NAME AS SERVER_NAME, SV.ID AS SERVER_ID, SV.WSEMAIL, PS.ID AS PRODUCT_SERVER_ID,
			       PS.BASE_URL, PS.PATH
              FROM PRODUCT PR
              LEFT JOIN PRODUCT_SERVER PS ON PS.PRODUCT_ID = PR.ID
              LEFT JOIN SERVER SV ON SV.ID = PS.SERVER_ID
             WHERE PR.ID = :PRODUCT
               AND PS.USETYPE = 'C'
               AND PS.IS_ACTIVE = 'Y'
               AND PR.ORGANIZATION_ID = :ORGANIZATION)
        WHERE 1=1 
        $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function inactiveReleasesFix($projectStatus) {
        $query = <<<SQL
        
        UPDATE RELEASE
           SET IS_ACTIVE = 'Y'
         WHERE ID IN (  SELECT REL.ID FROM RELEASE REL
                          JOIN PROJECT PRJ ON REL.PROJECT_ID  = PRJ.ID
                          JOIN TASK_STATUS TKS ON PRJ.PROJECT_STATUS = TKS.ID
                         WHERE REL.IS_ACTIVE = 'N'
                           AND REL.STATUS = 'H'
                           AND TKS.CONTROL <> '$projectStatus')
SQL;

      $this->getEntityManager()->getConnection()->executeQuery($query);
    }

    public function getAllTaskByRelease($releaseId){
        $query = <<<SQL
			   SELECT TAS.ID
                 FROM RELEASE REL
                 JOIN SUBRELEASE SRE ON (SRE.RELEASE_ID = REL.ID)
                 JOIN PROJECT PRO ON (PRO.ID = SRE.PROJECT_ID)
                 JOIN TASK TAS ON (TAS.PROJECT = PRO.ID)
                WHERE REL.ID = :RELEASE
                  AND TAS.STATUS = 'A'
             GROUP BY TAS.ID
SQL;
        $params = array(
            'RELEASE' => $releaseId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getAllIssuesByReleaseToSendMail($releaseId, $isHomologatedIssueFilter = false, $finalTaskStatusId = null) {
        $homologatedIssueFilter = '';
        $homologatedTaskStatus  = Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED;
        if($isHomologatedIssueFilter) {
            $homologatedIssueFilter = "JOIN TASK_STATUS TSI ON TSI.ID = ISS.TASK_STATUS AND NVL(TSI.CONTROL, ' ') <> '$homologatedTaskStatus'";
            if($finalTaskStatusId) {
                $homologatedIssueFilter .= " AND NVL(TSI.ID, ' ') <> $finalTaskStatusId";
            }
        }
        $query = <<<SQL
		   SELECT ISS.ID,RTRIM(XMLAGG(XMLELEMENT(E, TAK.ID, ',').EXTRACT('//text()') ORDER BY TAK.ID).GETCLOBVAL(), ',') AS TASKS,
		          (SELECT RTRIM(XMLAGG(XMLELEMENT(E, SRE.PROJECT_ID, ',').EXTRACT('//text()') ORDER BY SRE.PROJECT_ID).GETCLOBVAL(), ',') AS PROJECTS
                    FROM RELEASE REL
                    JOIN SUBRELEASE SRE ON (SRE.RELEASE_ID = REL.ID)
                    WHERE REL.ID = :RELEASE) AS PROJECTS,ORG.IDORGSAAS
              FROM RELEASE REL
              JOIN SUBRELEASE SRE ON (SRE.RELEASE_ID = REL.ID)
              JOIN PROJECT PRO ON (PRO.ID = SRE.PROJECT_ID)
              JOIN TASK TAS ON (TAS.PROJECT = PRO.ID AND TAS.STATUS = 'A')
              JOIN CLASSIFICATION CLA ON (CLA.ID = TAS.CLASSIFICATION AND (CLA.CONTROL IN (:ERROR_TYPE, :ERROR_MR_TYPE) OR CLA.CONTROL IS NULL))
              -- task release
              JOIN TASK_RELATIONSHIP TRI ON (TRI.TASK_RELATIONSHIP = TAS.ID AND TRI.TYPE =:RELATIONSHIP_ISSUE)
              JOIN TASK ISS ON (ISS.ID = TRI.TASK AND ISS.STATUS = 'A' AND ISS.IS_ISSUE = 'Y')
              $homologatedIssueFilter
              -- issue das tasks
              JOIN TASK_RELATIONSHIP TRT ON (TRT.TASK = ISS.ID AND TRT.TYPE =:RELATIONSHIP_ISSUE)
              JOIN TASK TAK ON (TAK.ID = TRT.TASK_RELATIONSHIP AND TAK.STATUS = 'A')
              JOIN CLASSIFICATION CLT ON (CLT.ID = TAK.CLASSIFICATION AND (CLT.CONTROL IN (:ERROR_TYPE, :ERROR_MR_TYPE) OR CLT.CONTROL IS NULL))
              -- tasks das issues
              JOIN PROJECT PRT ON (PRT.ID = TAK.PROJECT AND PRT.STATUS = 'A')
              JOIN SUBRELEASE SUB ON (SUB.PROJECT_ID = PRT.ID)
               -- cliente da issue
              JOIN ALLOCATION ALO ON (ALO.TYPE = 'S' AND ISS.ID = ALO.TASK)
              JOIN RESOURCE_T RES ON (ALO.RESOURCE_T = RES.ID) 
              JOIN CLIENTE CLI ON (RES.EXTERNAL_CODE = CLI.CDCLIENTE)
              JOIN ORGANIZACAO ORG ON (ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO)

             WHERE REL.ID = :RELEASE
               AND (SELECT COUNT(*) AS PROJECT_COUNT
                        FROM (SELECT PRO.ID
                                FROM TASK ISU
                                JOIN TASK_RELATIONSHIP TRL ON (ISU.ID = TRL.TASK AND TRL.TYPE = 'I')
                                JOIN TASK_RELATIONSHIP TRE ON ((TRE.TASK = TRL.TASK_RELATIONSHIP OR TRE.TASK_RELATIONSHIP = TRL.TASK_RELATIONSHIP) AND  TRE.TYPE = 'A')
                                JOIN TASK TAS ON TAS.ID = TRL.TASK_RELATIONSHIP
                                JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                                JOIN TASK_STATUS TST ON (TST.ID = PRO.PROJECT_STATUS AND TST.CONTROL NOT IN ('X', 'C'))
                                WHERE ISU.ID = ISS.ID
                            GROUP BY PRO.ID)) <= 1
          GROUP BY ISS.ID,ORG.IDORGSAAS
SQL;
        $params = array(
            'RELEASE' => $releaseId,
            'ERROR_TYPE' => TaskRelationshipEntities::ERROR_TYPE,
            'RELATIONSHIP_ISSUE' => TaskRelationshipEntities::RELATIONSHIP_ISSUE_TYPE,
            'ERROR_MR_TYPE' => TaskRelationshipEntities::ERROR_MR_TYPE
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getRelease($filters = "", $limit = null, $offset = null) {
        $query = <<<SQL
			SELECT (SELECT RTRIM(XMLAGG(XMLELEMENT(E, UA.USER_ID, ',').EXTRACT('//text()') ORDER BY UA.USER_ID).GETCLOBVAL(), ',')
                      FROM ZH_USER_ALLOCATION UA
                     WHERE UA.RELEASE_ID = RE.ID) AS USER_ID,
                   (SELECT RTRIM(XMLAGG(XMLELEMENT(E, UA.USER_ID, ',').EXTRACT('//text()') ORDER BY UA.USER_ID).GETCLOBVAL(), ',')
                      FROM ZH_USER_ALLOCATION UA
                     WHERE UA.RELEASE_ID = RE.ID) AS ZH_USER,
                   RE.SERVER_ID, RE.PRODUCT_NAME, RE.PRODUCT_VERSION, RE.PRODUCT_REVISION,
                   RE.PRODUCT_BUILD, RE.PRODUCT_SERVER_NAME, RE.WSEMAIL, RE.ID, RE.PRODUCT_ID, RE.NAME, RE.DESCRIPTION,
                   RE.IS_ACTIVE, RE.ORGANIZATION_ID, RE.CREATED, RE.CREATED_BY, RE.UPDATED, RE.UPDATED_BY, RE.STATUS,
                   RE.IS_FIX_RELEASE, RE.PRODUCT_SERVER_ID, RE.UPDATE_VERSION, RE.PROJECT_ID, RE.PROJECT_NAME,
                   RE.PROJECT_STATUS, RE.BASE_URL_PRODUCT_SERVER, RE.BASE_PATH_PRODUCT_SERVER, RE.VERSION, RE.REVISION, RE.BUILD, RE.USEGITAUTOMATICALLY
              FROM ( SELECT *
                       FROM (SELECT SE.ID AS SERVER_ID, PR.NAME AS PRODUCT_NAME, PR.VERSION AS PRODUCT_VERSION, PR.REVISION AS PRODUCT_REVISION,
                                    PR.BUILD AS PRODUCT_BUILD, SE.NAME AS PRODUCT_SERVER_NAME, SE.WSEMAIL, RE.ID, RE.PRODUCT_ID, RE.NAME, RE.DESCRIPTION,
                                    RE.IS_ACTIVE, RE.ORGANIZATION_ID, RE.CREATED, RE.CREATED_BY, RE.UPDATED, RE.UPDATED_BY, RE.STATUS, RE.IS_FIX_RELEASE,
                                    UA.USER_ID, UA.USER_ID AS ZH_USER, RE.PRODUCT_SERVER_ID, RE.UPDATE_VERSION, RE.PROJECT_ID, PJ.NAME AS PROJECT_NAME,
                                    PJ.STATUS AS PROJECT_STATUS,  PT.USEGITAUTOMATICALLY, PS.BASE_URL AS BASE_URL_PRODUCT_SERVER, PS.PATH AS BASE_PATH_PRODUCT_SERVER,
                                    CASE WHEN (RE.STATUS = 'H' OR RE.IS_FIX_RELEASE = 'Y') AND (RE.VERSION IS NOT NULL) THEN TO_CHAR(RE.VERSION) ELSE '-' END AS VERSION,
                                    CASE WHEN (RE.STATUS = 'H' OR RE.IS_FIX_RELEASE = 'Y') AND (RE.REVISION IS NOT NULL) THEN TO_CHAR(RE.REVISION) ELSE '-' END AS REVISION,
                                    CASE WHEN (RE.STATUS = 'H' OR RE.IS_FIX_RELEASE = 'Y') AND (RE.BUILD IS NOT NULL) THEN TO_CHAR(RE.BUILD) ELSE '-' END AS BUILD
                               FROM RELEASE RE
                               LEFT JOIN PROJECT PJ ON PJ.ID = RE.PROJECT_ID
                               LEFT JOIN PROJECT_TYPE PT ON PT.ID  = PJ.PROJECT_TYPE
                              INNER JOIN PRODUCT PR ON PR.ID = RE.PRODUCT_ID
                               LEFT JOIN ZH_USER_ALLOCATION UA ON UA.RELEASE_ID = RE.ID
                               LEFT JOIN PRODUCT_SERVER PS ON PS.ID = RE.PRODUCT_SERVER_ID AND PS.USETYPE = 'C'
                               LEFT JOIN SERVER SE ON SE.ID = PS.SERVER_ID
                              WHERE RE.IS_ACTIVE <> 'D'
                              ORDER BY RE.ID)
                      WHERE 1=1
                   $filters
                   ) RE
             GROUP BY RE.SERVER_ID, RE.PRODUCT_NAME, RE.PRODUCT_VERSION, RE.PRODUCT_REVISION,
                      RE.PRODUCT_BUILD, RE.PRODUCT_SERVER_NAME, RE.WSEMAIL, RE.ID, RE.PRODUCT_ID, RE.NAME, RE.DESCRIPTION,
                      RE.IS_ACTIVE, RE.ORGANIZATION_ID, RE.CREATED, RE.CREATED_BY, RE.UPDATED, RE.UPDATED_BY, RE.STATUS,
                      RE.IS_FIX_RELEASE, RE.PRODUCT_SERVER_ID, RE.UPDATE_VERSION, RE.PROJECT_ID, RE.PROJECT_NAME,
                      RE.PROJECT_STATUS, RE.BASE_URL_PRODUCT_SERVER, RE.BASE_PATH_PRODUCT_SERVER, RE.VERSION, RE.REVISION, RE.BUILD, RE.USEGITAUTOMATICALLY
             ORDER BY RE.ID
SQL;
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);

        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

}
