<?php
namespace Zeedhi\DevOps\Model\Configurationmanager\Repositories;

use \Doctrine\ORM\EntityRepository;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskRelationship as TaskRelationshipEntities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Doctrine\DBAL\Connection;

class Subrelease extends EntityRepository {
    /**
     * @param $subReleaseId
     * @param $serverType
     * @return array
     */
    public function findServersBySubReleaseIdAndType($subReleaseId, $serverType) {
        $query = <<<SQL
			SELECT SV.NAME AS SERVER_NAME, SV.ID AS SERVER_ID, SV.WSEMAIL, PS.ID AS PRODUCT_SERVER_ID
              FROM SUBRELEASE SR
        INNER JOIN RELEASE RL ON RL.ID = SR.RELEASE_ID
        INNER JOIN PRODUCT PR ON PR.ID = RL.PRODUCT_ID
         LEFT JOIN PRODUCT_SERVER PS ON PS.PRODUCT_ID = PR.ID
         LEFT JOIN SERVER SV ON SV.ID = PS.SERVER_ID
             WHERE SR.ID = :SUBRELEASE
               AND PS.USETYPE = :TYPE
               AND (PS.IS_ACTIVE = 'Y' OR PS.ID IS NULL)
SQL;
        $params = array(
            'SUBRELEASE' => $subReleaseId,
            'TYPE'       => $serverType
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $productId
     * @return array
     */
    public function findTestOrPackageServersByProduct($productId, $filterString = '') {
        $query = <<<SQL
        SELECT * FROM (
			SELECT S.NAME AS SERVER_NAME, S.ID AS SERVER_ID,
                   S.WSEMAIL, P.ID AS PRODUCT_SERVER_ID,
			       P.BASE_URL, P.PATH
              FROM SERVER S
             INNER JOIN PRODUCT_SERVER P ON P.PRODUCT_ID = :PRODUCT
                                        AND P.SERVER_ID = S.ID
             WHERE P.USETYPE = 'T'
               AND P.IS_ACTIVE = 'Y')
        WHERE 1=1
        $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $subReleaseId
     * @return array
     */
    public function findProductFilesBySubReleaseId($subReleaseId) {
        $query = <<<SQL
			SELECT PF.*
              FROM SUBRELEASE SR
        INNER JOIN RELEASE RL ON RL.ID = SR.RELEASE_ID
        INNER JOIN PRODUCT PR ON PR.ID = RL.PRODUCT_ID
        INNER JOIN PRODUCT_FILES PF ON PF.PRODUCT_ID = PR.ID
             WHERE SR.ID = :SUBRELEASE
SQL;
        $params = array('SUBRELEASE' => $subReleaseId);

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getAllTaskBySubRelease($subReleaseId){
        $query = <<<SQL
			  SELECT TAS.ID
                FROM SUBRELEASE SRE
                JOIN PROJECT PRO ON (PRO.ID = SRE.PROJECT_ID)
                JOIN TASK TAS ON (TAS.PROJECT = PRO.ID)
               WHERE SRE.ID = :RELEASE
                 AND TAS.STATUS = 'A'
               GROUP BY TAS.ID
SQL;
        $params = array(
            'RELEASE' => $subReleaseId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

     public function getAllIssueBySubReleaseToSendMail($subReleaseId, $isHomologatedIssueFilter = false, $finalTaskStatusId = null) {
         $homologatedIssueFilter = '';
         $homologatedTaskStatus  = Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED;
         if($isHomologatedIssueFilter) {
             $homologatedIssueFilter = "JOIN TASK_STATUS TSI ON TSI.ID = ISS.TASK_STATUS AND NVL(TSI.CONTROL, ' ') <> '$homologatedTaskStatus'";
             if($finalTaskStatusId) {
                 $homologatedIssueFilter .= " AND NVL(TSI.ID, ' ') <> $finalTaskStatusId";
             }
         }
         $query = <<<SQL
		    SELECT ISS.ID, RTRIM(XMLAGG(XMLELEMENT(E, TAK.ID, ',').EXTRACT('//text()') ORDER BY TAK.ID).GETCLOBVAL(), ',') AS TASKS,
                  (SELECT RTRIM(XMLAGG(XMLELEMENT(E, SRE.PROJECT_ID, ',').EXTRACT('//text()') ORDER BY SRE.PROJECT_ID).GETCLOBVAL(), ',') AS PROJECTS
                     FROM SUBRELEASE SRE
                    WHERE SRE.ID = :RELEASE) AS PROJECTS,ORG.IDORGSAAS
              FROM SUBRELEASE SRE
              JOIN PROJECT PRO ON (PRO.ID = SRE.PROJECT_ID)
              JOIN TASK TAS ON (TAS.PROJECT = PRO.ID)
              JOIN CLASSIFICATION CLA ON (CLA.ID = TAS.CLASSIFICATION AND (CLA.CONTROL IN (:ERROR_TYPE, :ERROR_MR_TYPE) OR CLA.CONTROL IS NULL))
              -- task release
              JOIN TASK_RELATIONSHIP TRI ON (TRI.TASK_RELATIONSHIP = TAS.ID AND TRI.TYPE =:RELATIONSHIP_ISSUE)
              JOIN TASK ISS ON (ISS.ID = TRI.TASK AND ISS.STATUS = 'A' AND ISS.IS_ISSUE = 'Y')
              $homologatedIssueFilter
              -- issue das tasks
              JOIN TASK_RELATIONSHIP TRT ON (TRT.TASK = ISS.ID AND TRT.TYPE =:RELATIONSHIP_ISSUE)
              JOIN TASK TAK ON (TAK.ID = TRT.TASK_RELATIONSHIP AND TAK.STATUS = 'A')
              JOIN CLASSIFICATION CLT ON (CLT.ID = TAK.CLASSIFICATION AND (CLT.CONTROL IN (:ERROR_TYPE, :ERROR_MR_TYPE) OR CLT.CONTROL IS NULL))
              -- tasks das issues
              JOIN PROJECT PRT ON (PRT.ID = TAK.PROJECT AND PRT.STATUS = 'A')
              JOIN SUBRELEASE SUB ON (SUB.PROJECT_ID = PRT.ID )
               -- cliente da issue
              JOIN ALLOCATION ALO ON (ALO.TYPE = 'S' AND ISS.ID = ALO.TASK)
              JOIN RESOURCE_T RES ON (ALO.RESOURCE_T = RES.ID) 
              JOIN CLIENTE CLI ON (RES.EXTERNAL_CODE = CLI.CDCLIENTE)
              JOIN ORGANIZACAO ORG ON (ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO)

             WHERE SRE.ID = :RELEASE
               AND (SELECT COUNT(*) AS PROJECT_COUNT
                      FROM (SELECT PRO.ID
                              FROM TASK ISU
                              JOIN TASK_RELATIONSHIP TRL ON (ISU.ID = TRL.TASK AND TRL.TYPE = 'I')
                              JOIN TASK_RELATIONSHIP TRE ON ((TRE.TASK = TRL.TASK_RELATIONSHIP OR TRE.TASK_RELATIONSHIP = TRL.TASK_RELATIONSHIP) AND  TRE.TYPE = 'A')
                              JOIN TASK TAS ON TAS.ID = TRL.TASK_RELATIONSHIP
                              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                              JOIN TASK_STATUS TST ON (TST.ID = PRO.PROJECT_STATUS AND TST.CONTROL NOT IN ('X', 'C'))
                             WHERE ISU.ID = ISS.ID
                          GROUP BY PRO.ID)) <= 1
          GROUP BY ISS.ID,ORG.IDORGSAAS
SQL;
        $params = array(
            'RELEASE' => $subReleaseId,
            'ERROR_TYPE' => TaskRelationshipEntities::ERROR_TYPE,
            'RELATIONSHIP_ISSUE' => TaskRelationshipEntities::RELATIONSHIP_ISSUE_TYPE,
            'ERROR_MR_TYPE' => TaskRelationshipEntities::ERROR_MR_TYPE
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getSubrelease($filters = "",$organizationId, $limit = null, $offset = null) {
        $query = <<<SQL
			SELECT (SELECT RTRIM(XMLAGG(XMLELEMENT(E, UA.USER_ID, ',').EXTRACT('//text()') ORDER BY UA.USER_ID).GETCLOBVAL(), ',')
                      FROM ZH_USER_ALLOCATION UA
                     WHERE UA.SUBRELEASE_ID = SR.ID) AS USER_ID,
                    (SELECT RTRIM(XMLAGG(XMLELEMENT(E, UA.USER_ID, ',').EXTRACT('//text()') ORDER BY UA.USER_ID).GETCLOBVAL(), ',')
                      FROM ZH_USER_ALLOCATION UA
                     WHERE UA.SUBRELEASE_ID = SR.ID) AS ZH_USER, 
                   SR.PRODUCT_VERSION, SR.PRODUCT_REVISION, SR.PRODUCT_BUILD, SR.RELEASE_NAME, SR.IS_FIX_RELEASE, SR.PRODUCT_ID, SR.PRODUCT_NAME,
                   SR.REPOSITORY_MASTER, SR.PRODUCT_SERVER_NAME,  SR.SERVER_ID, SR.BASE_URL_PRODUCT_SERVER, SR.BASE_PATH_PRODUCT_SERVER, SR.VERSION,
                   SR.REVISION, SR.BUILD, SR.ID, SR.RELEASE_ID, SR.NAME, SR.DESCRIPTION, SR.IS_ACTIVE, SR.ORGANIZATION_ID, SR.CREATED, SR.CREATED_BY, SR.UPDATED,
                   SR.UPDATED_BY, SR.STATUS, SR.URL_TO_TEST, SR.BRANCH_TO_DEPLOY, SR.PATH_TO_TEST, SR.PRODUCT_SERVER_ID, SR.PROJECT_ID, SR.PROJECT_NAME,
                   SR.PROJECT_STATUS, SR.PROJECT_PARENT, SR.WSEMAIL, SR.PRODUCT_SERVER_TYPE, SR.RELEASE_PRODUCT_SERVER_ID, SR.RELEASE_WSEMAIL,
                   SR.RELEASE_BASE_PATH_SERVER, SR.RELEASE_BASE_URL_SERVER, SR.PRODUCT_VERSION_ID, 
                   SR.PROJECT_PRODUCT_VERSION_ID, FINISH_SUBRELEASE_VALIDATED, SR.RELEASE_PRODUCT_SERVER_NAME, SR.USEGITAUTOMATICALLY, SR.UPDATE_VERSION
              FROM ( SELECT *
                       FROM (SELECT UA.USER_ID, UA.USER_ID AS ZH_USER, PR.VERSION AS PRODUCT_VERSION, PR.REVISION AS PRODUCT_REVISION, PR.BUILD AS PRODUCT_BUILD,
                                    RE.NAME AS RELEASE_NAME, RE.IS_FIX_RELEASE, RE.PRODUCT_ID, PR.NAME AS PRODUCT_NAME,
                                    RP.NAME AS REPOSITORY_MASTER, SE.NAME AS PRODUCT_SERVER_NAME, SE.WSEMAIL, SE.ID AS SERVER_ID,
                                    PS.BASE_URL AS BASE_URL_PRODUCT_SERVER, PS.PATH AS BASE_PATH_PRODUCT_SERVER, PS.USETYPE AS PRODUCT_SERVER_TYPE, 
                                    NVL(SS.NAME, SV.NAME) AS RELEASE_PRODUCT_SERVER_NAME,
                                    CASE WHEN SR.STATUS = 'H' AND SR.VERSION  IS NOT NULL
                                         THEN TO_CHAR(SR.VERSION)
                                         WHEN SR.PRODUCT_VERSION_ID IS NOT NULL
                                         THEN TO_CHAR(PV.VERSION)
                                         ELSE '-'
                                     END AS VERSION,
                                    CASE WHEN SR.STATUS = 'H' AND SR.REVISION IS NOT NULL
                                         THEN TO_CHAR(SR.REVISION)
                                         WHEN SR.PRODUCT_VERSION_ID IS NOT NULL
                                         THEN TO_CHAR(PV.REVISION)
                                         ELSE '-'
                                     END AS REVISION,
                                    CASE WHEN SR.STATUS = 'H' AND SR.BUILD    IS NOT NULL
                                         THEN TO_CHAR(SR.BUILD)
                                         ELSE '-'
                                     END AS BUILD,
                                     CASE WHEN PE.ID IS NULL THEN 'E'
                                          WHEN PE.IS_ACTIVE = 'N' THEN 'N'
                                          ELSE 'Y'
                                     END AS FINISH_SUBRELEASE_VALIDATED,
                                    SR.ID, SR.RELEASE_ID, SR.NAME, SR.DESCRIPTION, SR.IS_ACTIVE, SR.ORGANIZATION_ID, SR.CREATED, SR.CREATED_BY,
                                    SR.UPDATED, SR.UPDATED_BY, SR.STATUS, SR.URL_TO_TEST, SR.BRANCH_TO_DEPLOY, SR.PATH_TO_TEST, PS.ID AS PRODUCT_SERVER_ID,
                                    SR.PROJECT_ID, PJ.NAME AS PROJECT_NAME, PJ.STATUS AS PROJECT_STATUS, PJ.PARENT AS PROJECT_PARENT,
                                    NVL(PD.ID, PE.ID) AS RELEASE_PRODUCT_SERVER_ID, NVL(SS.WSEMAIL, SV.WSEMAIL) AS RELEASE_WSEMAIL, NVL(PD.PATH, PE.PATH) AS RELEASE_BASE_PATH_SERVER,
                                    NVL(PD.BASE_URL, PE.BASE_URL) AS RELEASE_BASE_URL_SERVER, SR.PRODUCT_VERSION_ID, PJ.PRODUCT_VERSION_ID AS PROJECT_PRODUCT_VERSION_ID, PT.USEGITAUTOMATICALLY, RE.UPDATE_VERSION
                               FROM SUBRELEASE SR
                              INNER JOIN RELEASE RE ON RE.ID = SR.RELEASE_ID
                               LEFT JOIN PROJECT PJ ON PJ.ID = SR.PROJECT_ID
                               LEFT JOIN PROJECT_TYPE PT ON PT.ID  = PJ.PROJECT_TYPE
                              INNER JOIN PRODUCT PR ON PR.ID = RE.PRODUCT_ID
                               LEFT JOIN REPOSITORY RP ON RP.PRODUCT = PR.ID AND RP.IS_MASTER = 'Y' AND RP.STATUS <> 'D'
                               LEFT JOIN PRODUCT_SERVER PS ON PS.ID = SR.PRODUCT_SERVER_ID AND PS.USETYPE = 'T'
                               LEFT JOIN SERVER SE ON SE.ID = PS.SERVER_ID
                               LEFT JOIN PRODUCT_SERVER PE ON PE.ID = RE.PRODUCT_SERVER_ID AND PE.USETYPE = 'C'
                               LEFT JOIN PRODUCT_VERSION PV ON SR.PRODUCT_VERSION_ID = PV.ID
                               LEFT JOIN SERVER SV ON SV.ID = PE.SERVER_ID
                               LEFT JOIN ZH_USER_ALLOCATION UA ON UA.SUBRELEASE_ID = SR.ID
                               LEFT JOIN PRODUCT_SERVER PD ON PD.ID = SR.RELEASE_PRODUCT_SERVER_ID
                               LEFT JOIN SERVER SS ON SS.ID = PD.SERVER_ID
                              WHERE SR.IS_ACTIVE <> 'D'
                                AND SR.ORGANIZATION_ID = :ORGANIZATION
                                )
                      WHERE 1=1
                    $filters 
                   ) SR
             GROUP BY SR.PRODUCT_VERSION, SR.PRODUCT_REVISION, SR.PRODUCT_BUILD, SR.RELEASE_NAME, SR.IS_FIX_RELEASE, SR.PRODUCT_ID, SR.PRODUCT_NAME,
                      SR.REPOSITORY_MASTER, SR.PRODUCT_SERVER_NAME, SR.WSEMAIL, SR.SERVER_ID, SR.BASE_URL_PRODUCT_SERVER, SR.BASE_PATH_PRODUCT_SERVER,
                      SR.VERSION, SR.REVISION, SR.BUILD, SR.ID, SR.RELEASE_ID, SR.NAME, SR.DESCRIPTION, SR.IS_ACTIVE, SR.ORGANIZATION_ID, SR.CREATED,
                      SR.CREATED_BY, SR.UPDATED, SR.UPDATED_BY, SR.STATUS, SR.URL_TO_TEST, SR.BRANCH_TO_DEPLOY, SR.PATH_TO_TEST, SR.PRODUCT_SERVER_ID,
                      SR.PROJECT_ID, SR.PROJECT_NAME, SR.PROJECT_STATUS, SR.PROJECT_PARENT, SR.PRODUCT_SERVER_TYPE, SR.RELEASE_PRODUCT_SERVER_ID,
                      RELEASE_WSEMAIL, SR.RELEASE_BASE_PATH_SERVER, SR.RELEASE_BASE_URL_SERVER, SR.PRODUCT_VERSION_ID, SR.PROJECT_PRODUCT_VERSION_ID,
                      FINISH_SUBRELEASE_VALIDATED, SR.RELEASE_PRODUCT_SERVER_NAME, SR.USEGITAUTOMATICALLY, SR.UPDATE_VERSION
             ORDER BY ID
SQL;

        $params = array(
            'ORGANIZATION' => $organizationId,
        );

        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query,$params);
    }

    public function findSubreleasePreviousVersionsNotFinalized($subReleaseId, $projectId, $version, $revision, $productVersionId) {
        $query = <<<SQL
			SELECT SR.ID, SR.NAME
              FROM SUBRELEASE SR
              JOIN PRODUCT_VERSION PV ON PV.ID = SR.PRODUCT_VERSION_ID
             WHERE SR.PROJECT_ID = :PROJECT_ID
               AND ((PV.VERSION <= :VERSION AND PV.REVISION <= :REVISION)
                    OR :PRODUCT_VERSION_ID IS NULL)
               AND SR.ID       <> :SUBRELEASE_ID
               AND SR.STATUS   <> :HOMOLOGATED
               AND SR.IS_ACTIVE = 'Y'
SQL;
        $params = array (
            'SUBRELEASE_ID'      => $subReleaseId,
            'PROJECT_ID'         => $projectId,
            'VERSION'            => $version,
            'REVISION'           => $revision,
            'PRODUCT_VERSION_ID' => $productVersionId,
            'HOMOLOGATED'        => \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease::HOMOLOGATED_STATUS
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findSubreleaseNotFinalizedByProject($subReleaseId, $projectId) {
        $query = <<<SQL
               SELECT ID, NAME
                 FROM SUBRELEASE
                WHERE PROJECT_ID = :PROJECT_ID
                  AND STATUS <> :HOMOLOGATED
                  AND ID <> :SUBRELEASE_ID
                  AND IS_ACTIVE = 'Y'
SQL;
        $params = array (
            'PROJECT_ID'    => $projectId,
            'SUBRELEASE_ID' => $subReleaseId,
            'HOMOLOGATED'   => \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease::HOMOLOGATED_STATUS
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findSubreleaseByProjectAndProductVersion($projectId, $productVersionId) {
        $query = <<<SQL
               SELECT ID, NAME, PRODUCT_VERSION_ID
                 FROM SUBRELEASE
                WHERE PROJECT_ID = :PROJECT_ID
                  AND PRODUCT_VERSION_ID = :PRODUCT_VERSION_ID
                  AND IS_ACTIVE = 'Y'
SQL;
        $params = array (
            'PROJECT_ID'         => $projectId,
            'PRODUCT_VERSION_ID' => $productVersionId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findSubreleaseByReleaseId($releaseId)
    {
        $query = <<<SQL
               SELECT ID
                 FROM SUBRELEASE
                WHERE RELEASE_ID = :RELEASE_ID
                  AND IS_ACTIVE = 'Y'
SQL;
        $params = array (
            'RELEASE_ID' => $releaseId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProductSubreleasesByStatus($productId, $subReleaseId, $status)
    {
        $query = <<<SQL
            SELECT S.ID, S.NAME
              FROM PRODUCT P
              JOIN RELEASE R ON R.PRODUCT_ID = P.ID
              JOIN SUBRELEASE S ON S.RELEASE_ID = R.ID
             WHERE S.ID <> :ID
               AND P.ID = :PRODUCT_ID
               AND R.IS_ACTIVE = 'Y'
               AND S.IS_ACTIVE = 'Y'
               AND S.STATUS IN (:STATUS)
          ORDER BY S.ID
SQL;
        $params = array (
            'ID'         => $subReleaseId,
            'PRODUCT_ID' => $productId,
            'STATUS'     => $status
        );
        $paramTypes = array(
            'STATUS' => Connection::PARAM_STR_ARRAY,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }
}
